#include <iostream>
#include <iomanip>
#include <cmath>

template <typename T>
struct Derive
{
    Derive(T & f, double h = 0.001) : _f(f), _h(h) {}
    double operator()(double x) const {
        return (_f(x+_h) - _f(x-_h))/(2*_h);
    }

    private:
        T & _f;
        double _h;
};

double carre(double x)
{
    return x*x;
}

int main()
{
    Derive d(sin);

    int i, n = 10;
    std::cout << "      x   sin'(x)    cos(x)" << std::endl;
    for (int i = 0; i<n; i++)
    {
        double x = double(i)/n;
        std::cout << std::setw(7) << x 
                  << std::setw(10) << d(x) 
                  << std::setw(10) << cos(x)
                  << std::endl;
    }
    std::cout << std::endl;

    Derive d2(carre);
 
    std::cout << "      x carre'(x)        2x" << std::endl;
    for (int i = 0; i<n; i++)
    {
        double x = double(i)/n;
        std::cout << std::setw(7) << x 
                  << std::setw(10) << d2(x) 
                  << std::setw(10) << 2.0*x
                  << std::endl;
    }

    return 0; 
}