#include <iostream>
#include "timer.hxx"

#include <thrust/device_vector.h>
#include <thrust/transform.h>
#include <thrust/functional.h>

void saxpy0a(double A, 
            const std::vector<double> &X, 
            std::vector<double> &Y)
{
    size_t n = X.size(), i;
    for (i=0; i<n; i++)
      Y[i] += A*X[i];
}

void saxpy0b(double A, 
            const std::vector<double> &X, 
            std::vector<double> &Y)
{
    size_t n = X.size(), i;
    #pragma omp parallel for
    for (i=0; i<n; i++)
      Y[i] += A*X[i];
}

void saxpy1(double A, 
            thrust::device_vector<double> &X, 
            thrust::device_vector<double> &Y)
{
  thrust::device_vector<double> temp(X.size());

  thrust::fill(temp.begin(), temp.end(), A);

  thrust::transform(X.begin(), X.end(),
                    temp.begin(), temp.begin(),
                    thrust::multiplies<double>());

  thrust::transform(temp.begin(), temp.end(),
                    Y.begin(), Y.begin(),
                    thrust::plus<double>());
}

struct saxpy_functor {

  const double _a;
  saxpy_functor(double a) : _a(a) {}

  __device__ 
  double operator() (const double &x, const double & y) const
  {
    return _a*x + y;
  }
};

void saxpy2(double A, 
            thrust::device_vector<double> &X, 
            thrust::device_vector<double> &Y)
{
  thrust::transform(X.begin(), X.end(), 
                    Y.begin(), Y.begin(),
                    saxpy_functor(A));
}

int main(int argc, char ** argv)
{
  size_t n = argc > 1 ? atoll(argv[1]) : 100000L;

  std::vector<double> Xref(n, 2.0);
  std::vector<double> Yref(n, 1.0);

  Timer T0a;
  T0a.start();
  saxpy0a(1.5, Xref, Yref);
  T0a.stop();
  std::cout << "saxpy (CPU):        time " << T0a.elapsed() << " s" << std::endl; 

  Timer T0b;
  T0b.start();
  saxpy0b(1.5, Xref, Yref);
  T0b.stop();
  std::cout << "saxpy (CPU openmp): time " << T0b.elapsed() << " s" << std::endl; 

  thrust::device_vector<double> X(n, 2.0);
  thrust::device_vector<double> Y(n, 1.0);

  Timer T1;
  T1.start();
  saxpy1(1.5, X, Y);
  T1.stop();
  std::cout << "saxpy GPU v1:       time " << T1.elapsed() << " s" << std::endl; 

  Timer T2;
  T2.start();
  saxpy2(1.5, X, Y);
  T2.stop();
  std::cout << "saxpy GPU v2:       time " << T2.elapsed() << " s" << std::endl; 
  
  return 1;
}
