#!  /usr/bin/env python3

import os, sys, subprocess, argparse, platform, shutil

def manage():

  parser = argparse.ArgumentParser()
  parser.add_argument('-m', '--mode', choices=['Release', 'Debug'], default='Release')
  args = parser.parse_args()

  myenv = os.environ.copy()
  
  p = platform.system()
  if p == 'Windows':
    gen = '-GNinja'
    compileCmd = ['ninja', 'install']
  else:
    gen = '-GUnix Makefiles'
    compileCmd = ['make', '--no-print-directory', 'install']

  baseDir = os.getcwd()
  srcDir = os.path.join(baseDir, 'src')

  buildDir = os.path.join(baseDir, 'build')
  installDir = os.path.join(baseDir, 'install')
 
  cmake_params = []
  cmake_params.append('-DCMAKE_INSTALL_PREFIX=' + installDir)
  cmake_params.append('-DCMAKE_BUILD_TYPE=' + args.mode)
  cmake_params.append(gen)

  if not os.path.exists(buildDir):
    os.makedirs(buildDir)

  if 'CC' in myenv:
    cmake_params.append('-DCMAKE_CUDA_HOST_COMPILER=' + myenv['CC'])
                       
  configureCmd = ['cmake'] + cmake_params + [srcDir]
  print('\n' + ' '.join(configureCmd) + '\n')
  err = subprocess.call(configureCmd, cwd=buildDir, env=myenv)
  if err == 0:
    err = subprocess.call(compileCmd, cwd=buildDir, env=myenv)

if __name__ == '__main__':
    manage()

