#include "calcul_pi.hxx"
#include<thrust/device_vector.h>
#include<thrust/count.h>
#include<thrust/copy.h>
#include<thrust/reduce.h>

__global__ void init(double * x, double dx, long n)
{
    long i = threadIdx.x + blockIdx.x * blockDim.x;
    double v;
    if (i < n) {
        v = i*dx;
        x[i] =  1.0/(1.0 + v*v);
    }
}

double calcul_pi_thrust1(long n)
{
   thrust::device_vector<double> d(n);
   double dx = 1.0/(n-1);

   double * dptr = thrust::raw_pointer_cast(d.data());
   init<<<(n+256)/256, 256>>>(dptr, dx, n);

   double sum = thrust::reduce(d.begin(), d.end());

   return 4*sum/n;
}


#include <thrust/for_each.h>
#include <thrust/execution_policy.h>
#include <thrust/tuple.h>
#include <thrust/iterator/counting_iterator.h>

struct functor {

    functor(double dx) : _dx(dx) {}
    template <typename Tuple>
    __device__
    void operator()(Tuple t) const {
        double x = _dx * (double) thrust::get<0>(t);
        thrust::get<1>(t) = 1.0/(1.0 + x*x);
    }

    double _dx;
};

double calcul_pi_thrust2(long n)
{
    thrust::device_vector<double> d(n);
    double dx = 1.0/(n-1);
    thrust::counting_iterator<unsigned int> first(0);
    thrust::counting_iterator<unsigned int> last = first + n;

    thrust::for_each(    
    thrust::make_zip_iterator(thrust::make_tuple(first, d.begin())), 
    thrust::make_zip_iterator(thrust::make_tuple(last, d.end())), 
    functor(dx));

    double sum = thrust::reduce(d.begin(), d.end());

    return 4*sum/n;
}
